library(shiny)

# Define server logic required to plot various variables against mpg
shinyServer(function(input, output) {
  
  two_samp_t_power <- function(alpha,n1,n2,s1,s2,xbar_diff,sides)
  {
    Sx1x2 <- sqrt(s1^2/n1+s2^2/n2)
    delta <- xbar_diff/Sx1x2
    dof <- (s1^2/n1+s2^2/n2)^2/(((s1^2/n1)^2/(n1-1))+((s2^2/n2)^2/(n2-1)))
    if (sides==2){
      talphaL <- qt(alpha/2,df=dof)
      talphaR <- qt(1-alpha/2,df=dof)
      power <- 1-pt(talphaR,df=dof,ncp=delta)+pt(talphaL,df=dof,ncp=delta)
    } else {
      talphaR <- qt(1-alpha,df=dof)
      power <- 1-pt(talphaR,df=dof,ncp=delta)
    }
    #delta<-(d/sigma)*sqrt(n)
    left<-qt(.001,df=dof)
    right<-qt(.999,df=dof,ncp=delta)
    x<-seq(left,right,.01)
    ta<-dt(x,df=dof)
    tb<-dt(x,df=dof,ncp=delta)
    if ( sides==1 ) {
      return(list(sides,x,ta,tb,delta,talphaR,power,delta))  
    } else {
      return(list(sides,x,ta,tb,delta,talphaL,talphaR,power,delta)) 
    }
  }
  
  
  formulaTextA <- reactive({
    b<-two_samp_t_power(input$alpha,input$n1,input$n2,input$s1,input$s2,input$xbar_diff,input$sides)
    if (b[[1]][1]==1) {
      paste("power =", round(b[[7]][1],digits=4))
    }else {
      paste("power =", round(b[[8]][1],digits=4))
    }
  })
  
  formulaTextB <- reactive({
    b<-two_samp_t_power(input$alpha,input$n1,input$n2,input$s1,input$s2,input$xbar_diff,input$sides)
    if (b[[1]][1]==1) {
      paste("critical t =", round(b[[6]][1],digits=4))
    }else {
      paste("critical t =", round(b[[6]][1],digits=4),"and",round(b[[7]][1],digits=4))
    }
  })
  
  formulaTextC <- reactive({
    b<-two_samp_t_power(input$alpha,input$n1,input$n2,input$s1,input$s2,input$xbar_diff,input$sides)
    if (b[[1]][1]==1) {
      paste("noncentrality param =", round(b[[8]][1],digits=4))
    }else {
      paste("noncentrality param =", round(b[[9]][1],digits=4))
    }
  })
  
  
  # Return the formula text for printing as a caption
  output$Cpower <- renderText({
    formulaTextA()
  })
  
  # Return the formula text for printing as a caption
  output$Ccrit <- renderText({
    formulaTextB()
  })
  
  # Return the formula text for printing as a caption
  output$Cncp <- renderText({
    formulaTextC()
  })
  
  output$powerPlot <- renderPlot({
    
    b<-two_samp_t_power(input$alpha,input$n1,input$n2,input$s1,input$s2,input$xbar_diff,input$sides)
    g<-plot(c(0,0),c(0,max(max(b[[3]]),max(b[[4]]))),type="l",
            col="black",xlab="Value of t Statistic",ylab="Value of Probability Density Function",
            xlim=c(b[[2]][1],b[[2]][length(b[[2]])])) 
    if ( b[[1]]==1 ) {
      
      w<-which.min(abs(b[[2]] - b[[6]][1]))
      
      #BLUE AREA
      g<-g+polygon(c(b[[2]][1],b[[2]][1:w],b[[2]][w]),
                   c(0,b[[4]][1:w],0),
                   col = "lightskyblue", border = NA)
      
      #RED AREA
      g<-g+polygon(c(b[[2]][w],b[[2]][w:length(b[[2]])],b[[2]][length(b[[2]])]),
                   c(b[[3]][length(b[[3]])],b[[3]][w:length(b[[3]])],b[[3]][length(b[[3]])]),
                   col = "lightpink1", border = NA)
      
      g<-g+lines(c(b[[6]],b[[6]]),c(0,max(max(b[[3]]),max(b[[4]]))), col="green")
      g<-g+lines(b[[2]],b[[3]],lwd=1,col="red") 
      g<-g+lines(b[[2]],b[[4]],lwd=1,pch=22, lty=2, col="blue")
      g<-g+lines(b[[2]],rep(0,length(b[[2]])),col="black")
      legend("topright", 
             cex=1,lwd=1, lty=c(1,2,1,NA,NA),
             pch = c(NA, NA, NA, 15,15), 
             col = c("red", "blue","green","lightskyblue","lightpink1"), 
             legend = c("Null", "Alternative","Critical t","Type II Error","Type I Error"))
      
    } else { 
      
      
      wl<-which.min(abs(b[[2]] - b[[6]][1]))
      w<-which.min(abs(b[[2]] - b[[7]][1]))
      
      #BLUE AREA
      g<-g+polygon(c(b[[2]][wl],b[[2]][wl:w],b[[2]][w]),
                   c(0,b[[4]][wl:w],0),col = "lightskyblue", border = NA)
      
      #LEFT RED AREA    
      g<-g+polygon(c(b[[2]][1],b[[2]][1:wl],b[[2]][wl]),
                   c(0,b[[3]][1:wl],0),col = "lightpink1", border = NA)
      
      #RIGHT RED AREA    
      g<-g+polygon(c(b[[2]][w],b[[2]][w:length(b[[2]])],b[[2]][length(b[[2]])]),
                   c(0,b[[3]][w:length(b[[3]])],0),
                   col = "lightpink1", border = NA)
      g<-g+lines(c(b[[6]],b[[6]]),c(0,max(max(b[[3]]),max(b[[4]]))), col="green")
      g<-g+lines(c(b[[7]],b[[7]]),c(0,max(max(b[[3]]),max(b[[4]]))), col="green")
      
      g<-g+lines(b[[2]],b[[3]],col="red") 
      g<-g+lines(b[[2]],b[[4]],pch=22, lty=2, col="blue")
      g<-g+lines(b[[2]],rep(0,length(b[[2]])),col="black")
      
      legend("topright", 
             cex=1,lwd=1, lty=c(1,2,1,NA,NA),
             pch = c(NA, NA, NA, 15,15), 
             col = c("red", "blue","green","lightskyblue","lightpink1"), 
             legend = c("Null", "Alternative","Critical t","Type II Error","Type I Error"))
      
    }
    
  })
  
  output$trendPlot <- renderPlot({
    if (input$sides==1){
      pow<-vector()
      count<-1
      for (i in seq(6,input$maxn,2)){
        pow[count]<-two_samp_t_power(input$alpha,i/2,i/2,input$s1,input$s2,input$xbar_diff,input$sides)[[7]] 
        count<-count+1
      }      
    } else {
      pow<-vector()
      count<-1
      
      for (i in seq(6,input$maxn,2)){
        pow[count]<-two_samp_t_power(input$alpha,i/2,i/2,input$s1,input$s2,input$xbar_diff,input$sides)[[8]] 
        count<-count+1
      }
    }
    gb<-plot(seq(6,input$maxn,2),pow,xlab="Total Sample Size (n1+n2)",ylab="Power",main="Plot of Trend (n1/n2=1)")
  })
  
  myout = reactive({
    
    if (input$sides==1){
      pow<-vector()
      count<-1
      for (i in seq(6,input$maxn,2)){
        pow[count]<-two_samp_t_power(input$alpha,i/2,i/2,input$s1,input$s2,input$xbar_diff,input$sides)[[7]] 
        count<-count+1
      }      
    } else {
      pow<-vector()
      count<-1
      
      for (i in seq(6,input$maxn,2)){
        pow[count]<-two_samp_t_power(input$alpha,i/2,i/2,input$s1,input$s2,input$xbar_diff,input$sides)[[8]] 
        count<-count+1
      }
    }
    
    dldata<-cbind(seq(6,input$maxn,2),pow)
    return(dldata)
  })
  
  
  output$downloadData <- downloadHandler(
    filename = function() { paste(Sys.Date(), '.csv', sep='') },
    content = function(file) {
      write.csv(myout(), file)})
  
})
